------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                          Ada Modeling Framework                          --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2012, Vadim Godunko <vgodunko@gmail.com>                     --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3305 $ $Date: 2012-11-09 13:39:11 +0100 (Fri, 09 Nov 2012) $
------------------------------------------------------------------------------
with Ada.Containers.Indefinite_Hashed_Maps;

with League.Holders;

with AMF.CMOF.Properties;

package AMF.Internals.Objects is

   pragma Preelaborate;

   type Get_Operation is
     access function
             (Object   : League.Holders.Holder;
              Property : not null AMF.CMOF.Properties.CMOF_Property_Access)
                return League.Holders.Holder;

   type Set_Operation is
     access procedure
             (Object   : in out League.Holders.Holder;
              Property : not null AMF.CMOF.Properties.CMOF_Property_Access;
              Value    : League.Holders.Holder);

   type Object_Type_Descriptor is record
      Get : Get_Operation;
      Set : Set_Operation;
   end record;

   type Object_Type_Decriptor_Access is access all Object_Type_Descriptor;

   function Hash (Item : League.Holders.Tag) return Ada.Containers.Hash_Type;

   package Tag_Descriptor_Maps is
     new Ada.Containers.Indefinite_Hashed_Maps
          (League.Holders.Tag,
           Object_Type_Decriptor_Access,
           Hash,
           League.Holders."=");

   Registry : Tag_Descriptor_Maps.Map;

end AMF.Internals.Objects;
