------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                          Ada Modeling Framework                          --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2011-2012, Vadim Godunko <vgodunko@gmail.com>                --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2990 $ $Date: 2012-05-16 09:25:25 +0200 (Wed, 16 May 2012) $
------------------------------------------------------------------------------
with AMF.Elements;
with AMF.Extents;
with AMF.Factories;

package AMF.Stores is

   pragma Preelaborate;

   type Store is limited interface
     and AMF.Extents.Extent
     and AMF.Factories.Factory;

   type Store_Access is access all Store'Class;
   for Store_Access'Storage_Size use 0;

   ----------------------
   --  AMF Extensions  --
   ----------------------

   not overriding function Get_Id
    (Self    : not null access Store;
     Element : not null AMF.Elements.Element_Access)
       return League.Strings.Universal_String is abstract;
   --  Returns identifier of the element inside the extent.

   not overriding procedure Set_Id
    (Self    : not null access Store;
     Element : not null AMF.Elements.Element_Access;
     Id      : League.Strings.Universal_String) is abstract;
   --  Sets identifier of the element inside the extent.

   not overriding function Get_Factory
    (Self          : not null access Store;
     Metamodel_URI : League.Strings.Universal_String)
       return AMF.Factories.Factory_Access is abstract;
   --  Returns factory for the specified URI of metamodel.

end AMF.Stores;
