------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                          Ada Modeling Framework                          --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2012, Vadim Godunko <vgodunko@gmail.com>                     --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2586 $ $Date: 2012-03-11 22:07:59 +0100 (Sun, 11 Mar 2012) $
------------------------------------------------------------------------------
with AMF.Elements.Collections;

package body AMF.Visitors.Containment is

   -----------
   -- Visit --
   -----------

   procedure Visit
    (Self    : in out Containment_Iterator'Class;
     Visitor : in out AMF.Visitors.Abstract_Visitor'Class;
     Extent  : not null access AMF.Extents.Extent'Class)
   is
      use type AMF.Elements.Element_Access;

      Elements : constant AMF.Elements.Collections.Set_Of_Element
        := Extent.Elements;
      Control  : Traverse_Control := Continue;

   begin
      for J in 1 .. Elements.Length loop
         if Elements.Element (J).Container = null then
            Self.Visit (Visitor, Elements.Element (J), Control);

            case Control is
               when Continue =>
                  null;

               when Abandon_Children =>
                  Control := Continue;

               when Abandon_Sibling =>
                  Control := Continue;

                  exit;

               when Terminate_Immediately =>
                  exit;
            end case;
         end if;
      end loop;
   end Visit;

end AMF.Visitors.Containment;
