------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                          Ada Modeling Framework                          --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2011-2012, Vadim Godunko <vgodunko@gmail.com>                --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 2864 $ $Date: 2012-04-13 12:08:07 +0200 (Fri, 13 Apr 2012) $
------------------------------------------------------------------------------
with AMF.Elements;
with AMF.Internals.Element_Collections;
with AMF.Internals.Helpers;
with AMF.Internals.Tables.UML_Attributes;
with AMF.Visitors.UML_Iterators;
with AMF.Visitors.UML_Visitors;

package body AMF.Internals.UML_Exception_Handlers is

   -------------------
   -- Enter_Element --
   -------------------

   overriding procedure Enter_Element
    (Self    : not null access constant UML_Exception_Handler_Proxy;
     Visitor : in out AMF.Visitors.Abstract_Visitor'Class;
     Control : in out AMF.Visitors.Traverse_Control) is
   begin
      if Visitor in AMF.Visitors.UML_Visitors.UML_Visitor'Class then
         AMF.Visitors.UML_Visitors.UML_Visitor'Class
          (Visitor).Enter_Exception_Handler
            (AMF.UML.Exception_Handlers.UML_Exception_Handler_Access (Self),
           Control);
      end if;
   end Enter_Element;

   -------------------
   -- Leave_Element --
   -------------------

   overriding procedure Leave_Element
    (Self    : not null access constant UML_Exception_Handler_Proxy;
     Visitor : in out AMF.Visitors.Abstract_Visitor'Class;
     Control : in out AMF.Visitors.Traverse_Control) is
   begin
      if Visitor in AMF.Visitors.UML_Visitors.UML_Visitor'Class then
         AMF.Visitors.UML_Visitors.UML_Visitor'Class
          (Visitor).Leave_Exception_Handler
            (AMF.UML.Exception_Handlers.UML_Exception_Handler_Access (Self),
           Control);
      end if;
   end Leave_Element;

   -------------------
   -- Visit_Element --
   -------------------

   overriding procedure Visit_Element
    (Self     : not null access constant UML_Exception_Handler_Proxy;
     Iterator : in out AMF.Visitors.Abstract_Iterator'Class;
     Visitor  : in out AMF.Visitors.Abstract_Visitor'Class;
     Control  : in out AMF.Visitors.Traverse_Control) is
   begin
      if Iterator in AMF.Visitors.UML_Iterators.UML_Iterator'Class then
         AMF.Visitors.UML_Iterators.UML_Iterator'Class
          (Iterator).Visit_Exception_Handler
            (Visitor,
             AMF.UML.Exception_Handlers.UML_Exception_Handler_Access (Self),
             Control);
      end if;
   end Visit_Element;

   -------------------------
   -- Get_Exception_Input --
   -------------------------

   overriding function Get_Exception_Input
    (Self : not null access constant UML_Exception_Handler_Proxy)
       return AMF.UML.Object_Nodes.UML_Object_Node_Access is
   begin
      return
        AMF.UML.Object_Nodes.UML_Object_Node_Access
         (AMF.Internals.Helpers.To_Element
           (AMF.Internals.Tables.UML_Attributes.Internal_Get_Exception_Input
             (Self.Element)));
   end Get_Exception_Input;

   -------------------------
   -- Set_Exception_Input --
   -------------------------

   overriding procedure Set_Exception_Input
    (Self : not null access UML_Exception_Handler_Proxy;
     To   : AMF.UML.Object_Nodes.UML_Object_Node_Access) is
   begin
      AMF.Internals.Tables.UML_Attributes.Internal_Set_Exception_Input
       (Self.Element,
        AMF.Internals.Helpers.To_Element
         (AMF.Elements.Element_Access (To)));
   end Set_Exception_Input;

   ------------------------
   -- Get_Exception_Type --
   ------------------------

   overriding function Get_Exception_Type
    (Self : not null access constant UML_Exception_Handler_Proxy)
       return AMF.UML.Classifiers.Collections.Set_Of_UML_Classifier is
   begin
      return
        AMF.UML.Classifiers.Collections.Wrap
         (AMF.Internals.Element_Collections.Wrap
           (AMF.Internals.Tables.UML_Attributes.Internal_Get_Exception_Type
             (Self.Element)));
   end Get_Exception_Type;

   ----------------------
   -- Get_Handler_Body --
   ----------------------

   overriding function Get_Handler_Body
    (Self : not null access constant UML_Exception_Handler_Proxy)
       return AMF.UML.Executable_Nodes.UML_Executable_Node_Access is
   begin
      return
        AMF.UML.Executable_Nodes.UML_Executable_Node_Access
         (AMF.Internals.Helpers.To_Element
           (AMF.Internals.Tables.UML_Attributes.Internal_Get_Handler_Body
             (Self.Element)));
   end Get_Handler_Body;

   ----------------------
   -- Set_Handler_Body --
   ----------------------

   overriding procedure Set_Handler_Body
    (Self : not null access UML_Exception_Handler_Proxy;
     To   : AMF.UML.Executable_Nodes.UML_Executable_Node_Access) is
   begin
      AMF.Internals.Tables.UML_Attributes.Internal_Set_Handler_Body
       (Self.Element,
        AMF.Internals.Helpers.To_Element
         (AMF.Elements.Element_Access (To)));
   end Set_Handler_Body;

   ------------------------
   -- Get_Protected_Node --
   ------------------------

   overriding function Get_Protected_Node
    (Self : not null access constant UML_Exception_Handler_Proxy)
       return AMF.UML.Executable_Nodes.UML_Executable_Node_Access is
   begin
      return
        AMF.UML.Executable_Nodes.UML_Executable_Node_Access
         (AMF.Internals.Helpers.To_Element
           (AMF.Internals.Tables.UML_Attributes.Internal_Get_Protected_Node
             (Self.Element)));
   end Get_Protected_Node;

   ------------------------
   -- Set_Protected_Node --
   ------------------------

   overriding procedure Set_Protected_Node
    (Self : not null access UML_Exception_Handler_Proxy;
     To   : AMF.UML.Executable_Nodes.UML_Executable_Node_Access) is
   begin
      AMF.Internals.Tables.UML_Attributes.Internal_Set_Protected_Node
       (Self.Element,
        AMF.Internals.Helpers.To_Element
         (AMF.Elements.Element_Access (To)));
   end Set_Protected_Node;

end AMF.Internals.UML_Exception_Handlers;
