------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                           SQL Database Access                            --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2011-2012, Vadim Godunko <vgodunko@gmail.com>                --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 3533 $ $Date: 2012-12-05 10:25:25 +0100 (Wed, 05 Dec 2012) $
------------------------------------------------------------------------------
--  This package provides type and operations to handle set of parameters of
--  SQL statements.
------------------------------------------------------------------------------
private with Ada.Containers.Hashed_Maps;

with League.Strings;
with League.Holders;
private with League.Strings.Hash;

package Matreshka.Internals.SQL_Parameter_Sets is

   pragma Preelaborate;

   type Parameter_Set is tagged limited private;

   procedure Clear (Self : in out Parameter_Set);
   --  Clears set of parameters.

   function Number_Of_Named (Self : Parameter_Set) return Natural;
   --  Returns number of named parameters.

   function Number_Of_Positional (Self : Parameter_Set) return Natural;
   --  Returns number of positional parameters.

   function Has_Parameter
    (Self : Parameter_Set;
     Name : League.Strings.Universal_String) return Boolean;
   --  Returns True when set has specified parameter.

   function Index
    (Self : Parameter_Set;
     Name : League.Strings.Universal_String) return Integer;
   --  Returns Index of the first occurrence of the specified parameter.

   procedure Append
    (Self : in out Parameter_Set;
     Name : League.Strings.Universal_String);
   --  Appends parameter with specified name and assign next index to it.

   procedure Alias
    (Self : in out Parameter_Set;
     Name : League.Strings.Universal_String);
   --  Creates alias for parameter with specified name and assign next index
   --  to it.

   procedure Set_Value
    (Self  : in out Parameter_Set;
     Name  : League.Strings.Universal_String;
     Value : League.Holders.Holder);
   --  Sets value of the specified parameter. Do nothing when parameter are not
   --  registered.

   function Value
    (Self : Parameter_Set; Index : Positive) return League.Holders.Holder;
   --  Returns value of the specified positional parameter.

private

   package Universal_String_Maps is
     new Ada.Containers.Hashed_Maps
          (League.Strings.Universal_String,
           League.Holders.Holder,
           League.Strings.Hash,
           League.Strings."=",
           League.Holders."=");

   function Hash (Item : Positive) return Ada.Containers.Hash_Type;

   package Integer_Maps is
     new Ada.Containers.Hashed_Maps
          (Positive,
           League.Strings.Universal_String,
           Hash,
           "=",
           League.Strings."=");

   type Parameter_Set is tagged limited record
      Values : Universal_String_Maps.Map;
      Names  : Integer_Maps.Map;
   end record;

end Matreshka.Internals.SQL_Parameter_Sets;
