------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                           SQL Database Access                            --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2011, Vadim Godunko <vgodunko@gmail.com>                     --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 1517 $ $Date: 2011-02-24 00:09:21 +0100 (Thu, 24 Feb 2011) $
------------------------------------------------------------------------------
with Matreshka.Internals.SQL_Drivers.SQLite3.Databases;

package body Matreshka.Internals.SQL_Drivers.SQLite3.Factory is

   type SQLite3_Factory is new Abstract_Factory with null record;

   overriding function Create
    (Self : not null access SQLite3_Factory) return not null Database_Access;

   ------------
   -- Create --
   ------------

   overriding function Create
    (Self : not null access SQLite3_Factory) return not null Database_Access is
   begin
      return new Databases.SQLite3_Database;
   end Create;

   use type Interfaces.C.int;

   Factory : aliased SQLite3_Factory;

begin
   --  Initialize threadsafety.

   if sqlite3_config (SQLITE_CONFIG_SERIALIZED) /= SQLITE_OK then
      raise Program_Error
        with "SQLite3 doesn't support serialized threading mode";
   end if;

   Register (League.Strings.To_Universal_String ("SQLITE3"), Factory'Access);
end Matreshka.Internals.SQL_Drivers.SQLite3.Factory;
