------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                               Web Framework                              --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2015, Vadim Godunko <vgodunko@gmail.com>                     --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 5172 $ $Date: 2015-01-22 08:06:23 +0100 (Thu, 22 Jan 2015) $
------------------------------------------------------------------------------
private with Ada.Containers.Vectors;

with League.Strings;

with Servlet.HTTP_Cookies;

package Servlet.HTTP_Cookie_Sets is

   pragma Preelaborate;

   type Cookie_Set is tagged private
     with Iterator_Element  => Servlet.HTTP_Cookies.Cookie,
          Constant_Indexing => Element;

   function Element
    (Self  : Cookie_Set'Class;
     Index : Positive) return Servlet.HTTP_Cookies.Cookie;

   function Element
    (Self : Cookie_Set'Class;
     Name : League.Strings.Universal_String)
       return Servlet.HTTP_Cookies.Cookie;

   function Has_Element
    (Self : Cookie_Set'Class;
     Name : League.Strings.Universal_String) return Boolean;

   function Length (Self : Cookie_Set'Class) return Natural;

   procedure Append
    (Self   : in out Cookie_Set'Class;
     Cookie : Servlet.HTTP_Cookies.Cookie);

private

   package Vectors is
     new Ada.Containers.Vectors
          (Positive,
           Servlet.HTTP_Cookies.Cookie,
           Servlet.HTTP_Cookies."=");

   type Cookie_Set is tagged record
      Data : Vectors.Vector;
   end record;

end Servlet.HTTP_Cookie_Sets;
