------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                               Web Framework                              --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2014-2015, Vadim Godunko <vgodunko@gmail.com>                --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 5172 $ $Date: 2015-01-22 08:06:23 +0100 (Thu, 22 Jan 2015) $
------------------------------------------------------------------------------
--  Provides a convenient implementation of the ServletResponse interface that
--  can be subclassed by developers wishing to adapt the response from a
--  Servlet. This class implements the Wrapper or Decorator pattern. Methods
--  default to calling through to the wrapped response object.
------------------------------------------------------------------------------
with League.Strings;

with Servlet.Output_Streams;
with Servlet.Responses;

package Servlet.Response_Wrappers is

   pragma Preelaborate;

   type Servlet_Response_Wrapper
    (Response : not null access Servlet.Responses.Servlet_Response'Class) is
       limited new Servlet.Responses.Servlet_Response with null record;

   overriding function Get_Output_Stream
    (Self : Servlet_Response_Wrapper)
       return
         not null access Servlet.Output_Streams.Servlet_Output_Stream'Class;

   overriding procedure Set_Character_Encoding
    (Self     : in out Servlet_Response_Wrapper;
     Encoding : League.Strings.Universal_String);

   overriding procedure Set_Content_Type
    (Self : in out Servlet_Response_Wrapper;
     To   : League.Strings.Universal_String);

end Servlet.Response_Wrappers;
